@@ -1,54 +1,68 @@
 import asyncio
 
 from pychonet.lib.const import (ENL_GETMAP, ENL_MANUFACTURER, ENL_PORT,
-                                ENL_SETMAP, ENL_UID, GET, MESSAGE_TIMEOUT)
+                                ENL_SETMAP, ENL_UID, GET, MESSAGE_TIMEOUT, ENL_STATMAP)
 from pychonet.lib.epc_functions import EPC_SUPER_FUNCTIONS
 from pychonet.lib.functions import TIDError, buildEchonetMsg, decodeEchonetMsg
 
 
 class ECHONETAPIClient:
-    def __init__(self, server, loop):
+    def __init__(self, server):
         self._server = server
-        self._loop = loop
         self._server.subscribe(self.echonetMessageReceived)
         self._state = {}
         self._next_tx_tid = 0x0000
         self._message_list = []
         self._message_timeout = MESSAGE_TIMEOUT
         self._debug_flag = False
+        self._update_callbacks = {}
 
     async def echonetMessageReceived(self, raw_data, addr):
+        updated = False
         host = addr[0]
         processed_data = decodeEchonetMsg(raw_data)
+        tid_found = processed_data["TID"] in self._message_list
+        isPush = not tid_found
         if self._debug_flag:
             print(f"Echonet Message Received - Processed data is {processed_data}")
-        # if we get duplicate packets that have already been processed then dont worry about the message list.
-        # but still process them regardless.
-        if processed_data["TID"] in self._message_list:
-            self._message_list.remove(processed_data["TID"])
+        seojgc = processed_data["SEOJGC"]
+        seojcc = processed_data["SEOJCC"]
+        seojci = processed_data["SEOJCI"]
+        key = f"{host}-{seojgc}-{seojcc}-{seojci}"
         # handle discovery message response
         for opc in processed_data["OPC"]:
-            seojgc = processed_data["SEOJGC"]
-            seojcc = processed_data["SEOJCC"]
-            seojci = processed_data["SEOJCI"]
             epc = opc["EPC"]
-            if seojgc == 14 and seojcc == 240 and epc == 0xD6:
+            if seojgc == 14 and seojcc == 240 and epc == 0xD6: # process discovery data
                 await self.process_discovery_data(host, opc)
-            else:
-                if epc == ENL_SETMAP:
-                    map = EPC_SUPER_FUNCTIONS[epc](opc["EDT"])
-                    self._state[host]["instances"][seojgc][seojcc][seojci][epc] = map
-                elif epc == ENL_GETMAP:
+            elif host not in self._state: # echonet packet arrived we dont know about
+                if self._debug_flag:
+                    print(f"Unknown ECHONETLite node has been identified at {host} - discovery packet fired")
+                await self.discover(host)
+            else: # process each EPC in order
+                if epc == ENL_SETMAP or epc == ENL_GETMAP or epc == ENL_STATMAP:
                     map = EPC_SUPER_FUNCTIONS[epc](opc["EDT"])
                     self._state[host]["instances"][seojgc][seojcc][seojci][epc] = map
+#                elif epc == ENL_GETMAP:
+#                    map = EPC_SUPER_FUNCTIONS[epc](opc["EDT"])
+#                    self._state[host]["instances"][seojgc][seojcc][seojci][epc] = map
                 elif epc in (ENL_UID, ENL_MANUFACTURER):
                     self._state[host]["instances"][seojgc][seojcc][seojci][
                         epc
                     ] = EPC_SUPER_FUNCTIONS[epc](opc["EDT"])
                 else:
-                    self._state[host]["instances"][seojgc][seojcc][seojci][epc] = opc[
-                        "EDT"
-                    ]
+                    if (epc not in self._state[host]["instances"][seojgc][seojcc][seojci] or self._state[host]["instances"][seojgc][seojcc][seojci][epc] != opc["EDT"]):
+                        updated = True
+                    self._state[host]["instances"][seojgc][seojcc][seojci][epc] = opc["EDT"]
+
+        # Call update callback functions
+        if updated and key in self._update_callbacks:
+            for update_func in self._update_callbacks[key]:
+                await update_func(isPush)
+
+        # if we get duplicate packets that have already been processed then dont worry about the message list.
+        # but still process them regardless.
+        if tid_found:
+            self._message_list.remove(processed_data["TID"])
 
     async def discover(self, host="224.0.23.0"):
         return await self.echonetMessage(host, 0x0E, 0xF0, 0x00, GET, [{"EPC": 0xD6}])
@@ -79,7 +93,8 @@ class ECHONETAPIClient:
         self._message_list.append(tx_tid)
         self._server.send(payload, (host, ENL_PORT))
         for x in range(0, self._message_timeout):
-            await asyncio.sleep(0.01)
+            # Wait up to 20(0.1*200) seconds depending on the Echonet specifications.
+            await asyncio.sleep(0.1)
             # if tx_tid is not in message list then the message listener has received the message
             if tx_tid not in self._message_list:
                 # transaction sucessful remove from list
@@ -88,7 +103,7 @@ class ECHONETAPIClient:
 
     async def getAllPropertyMaps(self, host, eojgc, eojcc, eojci):
         return await self.echonetMessage(
-            host, eojgc, eojcc, eojci, GET, [{"EPC": ENL_GETMAP}, {"EPC": ENL_SETMAP}]
+            host, eojgc, eojcc, eojci, GET, [{"EPC": ENL_STATMAP}, {"EPC": ENL_GETMAP}, {"EPC": ENL_SETMAP}]
         )
 
     async def getIdentificationInformation(self, host, eojgc, eojcc, eojci):
@@ -121,6 +136,9 @@ class ECHONETAPIClient:
                         self._state[host]["instances"][eojgc][eojcc].keys()
                     ):
                         self._state[host]["instances"][eojgc][eojcc][eojci] = {}
+                        self._state[host]["instances"][eojgc][eojcc][eojci].update(
+                            {ENL_STATMAP: []}
+                        )
                         self._state[host]["instances"][eojgc][eojcc][eojci].update(
                             {ENL_SETMAP: []}
                         )
@@ -128,3 +146,9 @@ class ECHONETAPIClient:
                             {ENL_GETMAP: []}
                         )
             self._state[host]["discovered"] = True
+
+    def register_async_update_callbacks(self, host, eojgc, eojcc, eojci, fn):
+        key = f"{host}-{eojgc}-{eojcc}-{eojci}"
+        if key not in self._update_callbacks:
+            self._update_callbacks[key] = []
+        self._update_callbacks[key].append(fn)
